package PureMessage::Database::Config;
use strict;
use warnings;

use Carp;
use PureMessage::Config qw(expand_filename);
use ActiveState::Config qw(:path);
use base qw(Exporter);

our $VERSION = '6.000';
our @EXPORT_OK = qw(cmpdsn);

# functions

sub cmpdsn ($$) {
    sub xdsn {
        my $dsn = shift;
        my(@dsn) = sort split(/;/, lc($dsn));
        return join(';', $dsn);
    }
      
    return xdsn($_[0]) cmp xdsn($_[1]);
}

# methods

sub new {
    my($class, %opt) = @_;
    my $cf = PureMessage::Config->new(%opt);
    bless {
	__conf__ => $cf,
    }, ref($class) || $class;
}

sub save {
    my $conf = delete $_[0]->{__conf__} or return;
    return $conf->save;
}

sub unlock {
    my $conf = delete $_[0]->{__conf__} or return;
    return $conf->unlock;
}

sub store_value {
    my $self = shift;
    my $store_id = shift;
    my $store_node = path_node($self->{__conf__}->root, "store#$store_id");
    croak "Can't find information for store $store_id in pmx.conf"
	unless $store_node;
    return path_value($store_node, @_);
}

sub update_dsn {
    my($self, $dsn, $store_id, $overwrite) = @_;
    $store_id ||= 'pmx_quarantine'; # XXX get rid of this
    my $path = "store#${store_id}.dsn";
    my $root = $self->{__conf__}->root;
    return 0 if((!$overwrite) && path_value($root, $path));
    return 0 if(
        path_value($root, $path) &&
        !cmpdsn(path_value($root, $path), $dsn)
    );
    path_value($root, $path, $dsn);
    return 1;
}

sub stores {
    my ($self) = @_; 
    my $config = $self->{__conf__}->as_hash();
    my %stores;

    # The 'pmx_quarantine_store' name only exists here
    $stores{pmx_quarantine_store} = 'pmx_quarantine';
    foreach my $store_type qw/pmx_prd_store pmx_resources_store/ {
        my $store_name;
        if ($store_name = $config->{$store_type}) {
           $stores{$store_type} = $store_name;
        }
    }
    return \%stores; 
}

1;

__END__

=head1 NAME

PureMessage::Database::Config - PMDB Configuration file interface

=head1 DESCRIPTION

The pmdb configuration file is used to store information about how
to connect to a relational database. See the L<pmx-config> man page for
information on the file format and options.

=head2 store_value($store_id, $option[, $value])

This single method can be used to quickly access a configuration option within
a store section. The first parameter is the store section to look in, the
second parameter is the option to lookup and the third, optional parameter is
the new value that the option will be set to. The option's value is returned.

=head2 update_dsn($dsn, [$store_id, [$overwrite]])

This method will update the DSN value of a store. By default, the
"pmx_quarantine" store is updated. If a DSN already exists for this store,
it will only be replaced it "$overwrite" is a TRUE value. If the value
is created or changed, 1 is returned; if not, 0 is returned.

=head2 stores()

Returns a hash to the referenced 'store' names in pmdb.conf
i.e. those for the resources, reports and quarantine.

The hash 'keys' being the store types, the 'values' the store name. 
The store name can then be used to retrieve the store's dsn value.

=head1 COPYRIGHT

Copyright (C) 2000-2007 Sophos Group. All rights reserved.

=head1 SEE ALSO

L<pmx-config>
